@@ -1,13 +1,13 @@
-## $Id: edit.py,v 1.133 2001/10/16 17:12:23 kjetilja Exp $
+## $Id: edit.py,v 1.140 2002/07/09 15:12:57 kjetilja Exp $
 
 ## System modules
 from gtk import *
 from gnome.ui import *
-import mimify, string, rfc822, cStringIO, sys, MimeWriter, mimetools, os.path
+import mimify, string, rfc822, cStringIO, sys, MimeWriter, os.path, time
 
 ## Local modules
 import cite, folderops, fileops, smileys, sendmail, mime, editor, builtineditor
-import externaleditor
+import externaleditor, pygmymimetools
 
 ## Mail composition modes
 NEW = 1
@@ -52,6 +52,61 @@ def verify_split(addr):
                 addr[idx] = addr[idx] +", "+ addr[idx+1]
                 del addr[idx+1]
 
+## Grabbed from email/Utils.py in later Python versions
+def formatdate(timeval=None, localtime=0, fromline=0):
+    # Note: we cannot use strftime() because that honors the locale and RFC
+    # 2822 requires that day and month names be the English abbreviations.
+    if timeval is None:
+        timeval = time.time()
+    if localtime:
+        now = time.localtime(timeval)
+        # Calculate timezone offset, based on whether the local zone has
+        # daylight savings time, and whether DST is in effect.
+        if time.daylight and now[-1]:
+            offset = time.altzone
+        else:
+            offset = time.timezone
+        hours, minutes = divmod(abs(offset), 3600)
+        # Remember offset is in seconds west of UTC, but the timezone is in
+        # minutes east of UTC, so the signs differ.
+        if offset > 0:
+            sign = '-'
+        else:
+            sign = '+'
+        zone = '%s%02d%02d' % (sign, hours, minutes / 60)
+    else:
+        now = time.gmtime(timeval)
+        # Timezone offset is always -0000
+        zone = '-0000'
+    if fromline:
+        # One format for the From line
+        return '%s %s %02d %02d:%02d:%02d %04d' % (
+            ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'][now[6]],
+            ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
+             'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'][now[1] - 1],
+            now[2], now[3], now[4], now[5], now[0])
+    else:
+        # Another format for regular date headers
+        return '%s, %02d %s %04d %02d:%02d:%02d %s' % (
+            ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'][now[6]],
+            now[2],
+            ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
+             'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'][now[1] - 1],
+            now[0], now[3], now[4], now[5], zone)
+
+## Date formatting for the reply/reply all citation start line
+def format_date_reply(timeval, localtime=0):
+    if timeval is None:
+        timeval = time.time()
+    now = time.localtime(timeval)
+    return '%s, %02d %s %04d at %02d:%02d' % (
+        ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'][now[6]],
+        now[2],
+        ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
+         'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'][now[1] - 1],
+        now[0], now[3], now[4])
+
+
 ##
 ##
 ## Edit/Compose window class
@@ -189,7 +244,7 @@ class EditWindow:
             if len(not_viewed) > 0:
                 for sm in not_viewed:
                     type = sm.gettype()
-                    name = mimify.mime_decode_header((sm.getparam('name') or\
+                    name = mimify.mime_decode_header((sm.getparam('name') or sm.getparam('filename') or\
                                               NONAME))[:40]
                     # Append attachment to list
                     self.attachments.insert(0, (name, type, sm))
@@ -269,7 +324,7 @@ class EditWindow:
             if len(not_viewed) > 0:
                 for sm in not_viewed:
                     type = sm.gettype()
-                    name = mimify.mime_decode_header((sm.getparam('name') or\
+                    name = mimify.mime_decode_header((sm.getparam('name') or sm.getparam('filename') or\
                                                       NONAME))[:40]
                     # Append attachment to list
                     self.attachments.insert(0, (name, type, sm))
@@ -310,7 +365,7 @@ class EditWindow:
         body = ''
         msg = self.msg
 
-        date = msg.getheader('date')
+        date = format_date_reply(int(folderops.date_to_epoch(msg)))
         frm = mimify.mime_decode_header(msg.getaddr('from')[0] or 'you')
 
         # Fill in the subject, to, date and body parts (reply to all)
@@ -326,7 +381,8 @@ class EditWindow:
         # Get all the relevant addresses from the accounts
         ownaddrs = []
         for a in self.prefs.accounts.keys():
-            username, emailaddr, sigfile, replyaddr = self.prefs.accounts[a]
+            username, emailaddr, sigfile, replyaddr, smtpserver, localsendmail \
+                      = self.prefs.accounts[a]
             if emailaddr != '':
                 ownaddrs.append(string.lower(emailaddr))
             if replyaddr != '':
@@ -837,18 +893,21 @@ class EditWindow:
         else:
             showfrom = 1
 
-        # Set the default account to use -- should be adaptive to
-        # the recipient address if replying to a mail -- later
-        self.account_name = self.prefs.accounts.keys()[0]
+        # Set the default account to use -- adapt to
+        # the recipient address if replying to a mail if REPLY/REPLYALL
+        if self.mode in (REPLY, REPLYALL):
+            self.account_name = self.get_account_name_for_reply()
+        else:
+            self.account_name = self.prefs.defacc
         
         # Determine which account is the default one
         e = self.prefs.accounts.keys()
         e.sort()
-        if self.prefs.defacc and len(e) > 1:
-            item = e.index(self.prefs.defacc)
-            self.account_name = self.prefs.defacc
+        if self.account_name and len(e) > 1:
+            item = e.index(self.account_name)
         else:
             item = 0
+            self.account_name = e[0]
 
         # From: entry
  	l = GtkLabel("From")
@@ -945,6 +1004,30 @@ class EditWindow:
             self.l2.show()
             self.e4.show()
 
+    ##
+    def get_account_name_for_reply(self):
+        import string
+        # Get all the relevant accounts corresponding to email address
+        ownacc = {}
+        for a in self.prefs.accounts.keys():
+            username, emailaddr, sigfile, replyaddr, smtpserver, localsendmail \
+                      = self.prefs.accounts[a]
+            if emailaddr != '':
+                ownacc[emailaddr] = a
+            if replyaddr != '':
+                ownacc[replyaddr] = a
+
+        # Get all to + cc address in the to-be-replied msg
+        to = self.msg.getaddrlist('To')
+        cc = self.msg.getaddrlist('Cc')
+        allAddr = map (lambda x: x[1], to + cc)
+        for addr in allAddr:
+            if addr in ownacc.keys():
+                return ownacc[addr]   # return the first matched emailAddr's
+                                        # accout name
+        return  self.prefs.defacc # else, the default account
+
+
 
     ## Toggle more or less information for headers
     def more_toggle(self, button=None):
@@ -1107,7 +1190,8 @@ class EditWindow:
 
         # Get the account we should use for sending the mail
         account = self.prefs.accounts[self.account_name]
-        username, emailaddr, sigfile, replyaddr = account
+        username, emailaddr, sigfile, replyaddr, smtpserver, localsendmail = account
+
 
         # Extract To and CC widget contents
         to = self.e1.get_text()
@@ -1159,14 +1243,16 @@ class EditWindow:
         headers = headers + 'X-Mailer: Pygmy (v%s)\n' % self.prefs.version
 
         # Set date header entry
-        headers = headers + 'Date: %s\n' % \
-                  mime_encode(time.strftime('%a, %d %b %Y %H:%M:%S %Z',
-                                            time.localtime(time.time())))
+        headers = headers + 'Date: %s\n' % formatdate(localtime=1)
 
         # Add the rest of the headers
         for h in self.headers.keys():
             headers = headers + '%s: %s\n' % (h, mime_encode(self.headers[h]))
 
+        charset = ''
+        if (string.find(self.prefs.compose_font,'8859-') > 0):
+  	  charset = 'iso-' + (self.prefs.compose_font[(string.find(self.prefs.compose_font,'8859-')):])
+
         # Message body
         msg = ''
         if len(self.attachments) > 0:
@@ -1187,7 +1273,7 @@ class EditWindow:
                     p.addheader('Content-Transfer-Encoding', enc)
                     inbody = cStringIO.StringIO(txt)
                     outbody = cStringIO.StringIO()
-                    mimetools.encode(inbody, outbody, enc)
+                    pygmymimetools.encode(inbody, outbody, enc)
                     txt = '\n' + outbody.getvalue()
                     del inbody, outbody
                     while string.find(txt, '\n.\n') != -1:
@@ -1197,9 +1283,11 @@ class EditWindow:
                 txt = self.privacy.handle_write(txt, self.send_mail)
                 if not txt: # Will be None if e.g. passphrase was wrong
                     return
-                b = p.startbody('text/plain')
+                if charset:
+                  b = p.startbody('text/plain' + '; charset="' + charset + '"') 
+                else:
+                  b = p.startbody('text/plain')
                 b.write(txt) 
-
             # Write all the attachments
             enc = 'base64'
             for name, type, sm in self.attachments:
@@ -1209,12 +1297,12 @@ class EditWindow:
                 b = p.startbody(type, [('name', filename)])
                 if sm != None:
                     # A submessage may be included (e.g. for forward)
-                    mimetools.encode(cStringIO.StringIO(sm.getbodytext()), b, enc)
+                    pygmymimetools.encode(cStringIO.StringIO(sm.getbodytext()), b, enc)
                     del sm
                 else:
                     # ... or just a file name reference (for new messages)
                     file = open(name)
-                    mimetools.encode(file, b, enc)
+                    pygmymimetools.encode(file, b, enc)
                     file.close()
             t.lastpart()
 
@@ -1232,10 +1320,13 @@ class EditWindow:
                 enc = 'quoted-printable'
                 inbody = cStringIO.StringIO(msg)
                 outbody = cStringIO.StringIO()
-                mimetools.encode(inbody, outbody, enc)
+                pygmymimetools.encode(inbody, outbody, enc)
                 msg = outbody.getvalue()
                 del inbody, outbody
-                headers = headers + "Content-type: text/plain\n"
+                if charset:
+                  headers = headers + "Content-Type: text/plain" + '; charset="' + charset + '"\n' 
+                else:
+                  headers = headers + "Content-Type: text/plain\n"
                 headers = headers + "Content-Transfer-Encoding: %s\n" % enc
                 # Must encode single dots on lines (quoted-printable)
                 while string.find(msg, '\n.\n') != -1:
@@ -1249,25 +1340,25 @@ class EditWindow:
         # Send mail to target host if immediate send was specified
         if immediate:
             target = 'sent-mail'
-            if self.prefs.usemailserver:
+            if smtpserver != '':
                 # Use direct connection to mail server
                 message = recp_tocc + headers + msg
-                if not sendmail.sendmail_server(self.prefs.serverentry, frm,
+                if not sendmail.sendmail_server(smtpserver, frm,
                                                 to+cc, message):
                     return 0
                 if bcc != []:
                     message = recp_bcc + headers + msg
-                    if not sendmail.sendmail_server(self.prefs.serverentry, frm,
+                    if not sendmail.sendmail_server(smtpserver, frm,
                                                     bcc, message):
                         return 0
-            elif self.prefs.usesendmail:
+            else:
                 # Use sendmail command at localhost
                 message = recp_tocc + headers + msg
-                if not sendmail.sendmail_cmd(self.prefs.sendmailentry, message):
+                if not sendmail.sendmail_cmd(localsendmail, message):
                     return 0                
                 if bcc != []:
                     message = recp_bcc + headers + msg
-                    if not sendmail.sendmail_cmd(self.prefs.sendmailentry, message):
+                    if not sendmail.sendmail_cmd(localsendmail, message):
                         return 0
             # Popup that the message was successfully sent/queued
             if self.prefs.confirmsend:
@@ -1283,7 +1374,7 @@ class EditWindow:
                 w.show()
             
         # Add a date header and the From line at the top of the message
-        prefix = 'From %s  %s\n' % (emailaddr, time.ctime(time.time()))
+        prefix = 'From %s  %s\n' % (emailaddr, formatdate(localtime=1, fromline=1))
 
         # Open the target folder and store the message there
         fo = open(self.prefs.folders+'/'+target, 'a')
