@@ -1,11 +1,11 @@
-## $Id: prefs.py,v 1.69 2001/10/10 12:13:52 kjetilja Exp $
+## $Id: prefs.py,v 1.72 2001/10/16 17:48:32 kjetilja Exp $
 
 ## System modules
 from gtk import *
 from gnome.ui import *
 from gnome.config import *
 from posixpath import split
-import os, glob
+import os, glob, sys
 
 ## Local import pwdmodules
 import folderops, fileops, fonts
@@ -53,16 +53,20 @@ class Preferences:
         self.accounts = {}
         self.filepaths = {}
         account_names = get_vector('pygmy/prefs/account_names')
+
         for name in account_names:
             self.accounts[name] = get_vector('pygmy/accounts/%s' % name)
             if self.accounts[name][0][0] == '_':
                 self.accounts[name] = map(lambda x: x[1:], self.accounts[name])
         self.defacc = get_string('pygmy/prefs/defacc') or ''
         
+        # Keep temporarily for consistency with older versions of Pygmy
         self.usemailserver = get_bool('pygmy/prefs/usemailserver') or 0
         self.serverentry = get_string('pygmy/prefs/serverentry') or "smtp"
         self.usesendmail = get_bool('pygmy/prefs/usesendmail') or 0
         self.sendmailentry = get_string('pygmy/prefs/sendmailentry') or "/usr/lib/sendmail"
+        # End of consistency check
+
         self.external_cmd = get_string('pygmy/prefs/external_cmd') or ""
         self.invoke_at_startup = get_bool('pygmy/prefs/invoke_at_startup') or 0
         self.mailnotify = get_bool('pygmy/prefs/mailnotify') or 0
@@ -117,7 +121,7 @@ class Preferences:
 
         self.more = get_bool('pygmy/prefs/moreheaders') or 0
 
-        self.version = "0.5.13"
+        self.version = "0.6.0"
 
         if launch_preferences:
             p = PreferencesWindow(self)
@@ -134,10 +138,9 @@ class Preferences:
         set_bool('pygmy/prefs/filter_start', self.filter_start) 
         set_bool('pygmy/prefs/gtkhtml', self.gtkhtml) 
         set_string('pygmy/prefs/defacc', self.defacc)
-        set_bool('pygmy/prefs/usemailserver', self.usemailserver) 
-        set_string('pygmy/prefs/serverentry', self.serverentry)
-        set_bool('pygmy/prefs/usesendmail', self.usesendmail)
-        set_string('pygmy/prefs/sendmailentry', self.sendmailentry)
+#       set_bool('pygmy/prefs/usemailserver', self.usemailserver) 
+#       set_string('pygmy/prefs/serverentry', self.serverentry)
+#       set_bool('pygmy/prefs/usesendmail', self.usesendmail)
         set_string('pygmy/prefs/external_cmd', self.external_cmd)
         set_bool('pygmy/prefs/invoke_at_startup', self.invoke_at_startup)
         set_bool('pygmy/prefs/mailnotify', self.mailnotify)
@@ -165,7 +168,7 @@ class Preferences:
         set_string('pygmy/prefs/spoolfile', self.spoolfile)
         set_string('pygmy/prefs/folders', self.folders)
 
-	# editor preferences
+	# Editor preferences
         set_bool('pygmy/prefs/usebuiltined', self.usebuiltined )
         set_bool('pygmy/prefs/useexternaled', not self.usebuiltined )
         set_string('pygmy/prefs/externaledentry', self.externaledentry )
@@ -173,7 +176,23 @@ class Preferences:
 
         clean_section('pygmy/accounts')
         set_vector('pygmy/prefs/account_names', self.accounts.keys())
+
         for name in self.accounts.keys():
+            # For consistency with older than 0.5.13 preferences file
+            if len(self.accounts[name]) == 4:
+                if self.usesendmail:
+                    self.accounts[name] = self.accounts[name] + \
+                                          ['', self.sendmailentry]
+                else:
+                    self.accounts[name] = self.accounts[name] + \
+                                          [self.serverentry, '']
+
+                self.usemailserver, self.sendmailentry, self.usesendmail,\
+                self.sendmailentry = 0, '', 0, ''
+            # End of consistency check
+
+            if self.accounts[name][4] != '': self.accounts[name][5] = '' 
+
             set_vector('pygmy/accounts/%s' % name, map(lambda x: '_'+ x,
                                                        self.accounts[name][:]))
         clean_section('pygmy/geometry')
@@ -182,14 +201,11 @@ class Preferences:
             vec = map(lambda x: str(x), self.size[index])
             set_vector('pygmy/geometry/win_%d' % index, vec)
             index = index + 1
-
         clean_section('pygmy/filepaths')
         set_vector('pygmy/prefs/default_filepaths', self.filepaths.keys())
         for name in self.filepaths.keys():
             set_string('pygmy/filepaths/%s' % name, self.filepaths[name] or '' )
-
         set_bool('pygmy/prefs/moreheaders', self.more)
-
         sync()
         
 
@@ -243,10 +259,6 @@ class PreferencesWindow:
             if self.entry.has_key(DEF_ACC):
                 del self.entry[DEF_ACC]
             self.prefs.accounts = self.entry.copy()
-            self.prefs.serverentry = self.serverentry.get_text()
-            self.prefs.usemailserver = self.usemailserver.get_active()
-            self.prefs.usesendmail = self.usesendmail.get_active()
-            self.prefs.sendmailentry = self.sendmailentry.gtk_entry().get_text()
             self.prefs.folders = self.direntry.gtk_entry().get_text()
             self.prefs.spoolfile = self.spoolentry.gtk_entry().get_text()
             self.prefs.mailnotify = self.notify.get_active()
@@ -275,10 +287,10 @@ class PreferencesWindow:
                 self.prefs.editorhints = self.editorhints.get_menu().get_active().get_data("id")
 
             # Ensure only one of mailserver or smtp is set
-            if self.prefs.usemailserver:
-                self.prefs.usesendmail = 0
-            if self.prefs.usesendmail:
-                self.prefs.usemailserver = 0
+            #if self.prefs.usemailserver:
+            #    self.prefs.usesendmail = 0
+            #if self.prefs.usesendmail:
+            #    self.prefs.usemailserver = 0
 
             # Check that the username and email address is entered
             if len(self.prefs.accounts) == 0:
@@ -354,14 +366,6 @@ class PreferencesWindow:
         label = GtkLabel('Incoming')
         notebook.append_page(frame, label)
         self.init_incoming(frame)
-
-        frame = GtkFrame('Outgoing')
-        frame.set_border_width(10)
-        frame.set_usize(400, 320)
-        frame.show()
-        label = GtkLabel('Outgoing')
-        notebook.append_page(frame, label)
-        self.init_outgoing(frame)
     
         if self.externaled_support:
             frame = GtkFrame('Editor')
@@ -410,22 +414,29 @@ class PreferencesWindow:
         if acc and len(e) > 0:
             if acc in e:
                 row = e.index(acc)
-                self.acc.set_background(row, GdkColor(25535,25535,32535))
+                self.acc.set_background(row, GdkColor(29535,29535,32535))
             else:
                 self.prefs.defacc = ''
 
 
     def acc_edit_cb(self, button, no):
         w = self.dialogs[button]
-        accname = w[5]
+        accname = w[8]
         if no == 0:
             del self.entry[accname]
-            self.entry[w[0].get_text()] = [
+
+            self.entry[accname] = [
                 w[1].get_text(),
                 w[2].get_text(),
                 w[4].gtk_entry().get_text(),
                 w[3].get_text(),
+                w[5].get_text(),
+                w[6].gtk_entry().get_text()
                 ]
+
+            if not w[7].get_active(): 
+                self.entry[accname][4] = '' # Flag that smtpserver is not used
+
             self.update_acc()
         elif no == 1:
             pass
@@ -471,6 +482,8 @@ class PreferencesWindow:
                 w[2].get_text(),
                 w[4].gtk_entry().get_text(),
                 w[3].get_text(),
+                w[5].get_text(),
+                w[6].gtk_entry().get_text(),
                 ]
             self.update_acc()
         del self.dialogs[button]
@@ -489,57 +502,120 @@ class PreferencesWindow:
         
 
     def acc_view(self, accname):
-        name, email, sigfile, reply = self.entry[accname]
-        t = GtkTable(11,1,0)
+        if len (self.entry[accname]) == 6:
+            name, email, sigfile, reply, smtpserver, sendmail = self.entry[accname]
+        else: # For compatibility with older versions of Pymy
+            name, email, sigfile, reply = self.entry[accname]
+            smtpserver, sendmail = '', ''
+        if smtpserver != '': usesmtpserver = 1
+        else: usesmtpserver = 0
+        if sendmail == '': sendmail = '/usr/lib/sendmail'
+
+        notebook = GtkNotebook()
+        notebook.set_tab_pos(POS_TOP)
+        notebook.show()
+
+        # Make tab for identity
+        frame = GtkFrame('Identity')
+        frame.set_border_width(10)
+        frame.set_usize(320, 300)
+        frame.show()
+        label = GtkLabel('Identity')
+        notebook.append_page(frame, label)
+
+        t = GtkTable(15,1,0)
         t.show()
+
 	la = GtkLabel("Account name:")
 	la.show()
+        la.set_alignment(0.0, 0.5)
 	accentry = GtkEntry()
         accentry.show()
         accentry.set_text(accname)
+
 	ln = GtkLabel("User name:")
 	ln.show()
+        ln.set_alignment(0.0, 0.5)
 	nameentry = GtkEntry()
         nameentry.show()
         if name != None:
             nameentry.set_text(name)
+
 	le = GtkLabel("Email address:")
 	le.show()
+        le.set_alignment(0.0, 0.5)
 	mailentry = GtkEntry()
         mailentry.show()
         if email != None:
             mailentry.set_text(email)
+
 	ls = GtkLabel("Signature file:")
 	ls.show()
+        ls.set_alignment(0.0, 0.5)
 	sigentry = GnomeFileEntry()
         sigentry.gnome_entry().set_history_id('signature')
         sigentry.gnome_entry().load_history()
         sigentry.show()
         if sigfile != None:
             sigentry.gtk_entry().set_text(sigfile)
+
 	lr = GtkLabel("Reply-to address:")
 	lr.show()
+        lr.set_alignment(0.0, 0.5)
 	replyentry = GtkEntry()
         replyentry.show()
         if reply != None:
             replyentry.set_text(reply)
-        t.attach(la, 0, 1, 0, 1, yoptions=0, ypadding=3)
-        t.attach(accentry, 0, 1, 1, 2, xpadding=40, yoptions=0,
-                 ypadding=3)
-        t.attach(ln, 0, 1, 2, 3, yoptions=0, ypadding=3)
-        t.attach(nameentry, 0, 1, 3, 4, xpadding=40, yoptions=0,
-                 ypadding=3)
-        t.attach(le, 0, 1, 4, 5, yoptions=0, ypadding=3)
-        t.attach(mailentry, 0, 1, 5, 6, xpadding=40, yoptions=0,
-                 ypadding=3)
-        t.attach(lr, 0, 1, 6, 7, yoptions=0, ypadding=3)
-        t.attach(replyentry, 0, 1, 7, 8, xpadding=40, yoptions=0,
-                 ypadding=3)
-        t.attach(ls, 0, 1, 8, 9, yoptions=0, ypadding=3)
-        t.attach(sigentry, 0, 1, 10, 11, xpadding=40, yoptions=0,
-                 ypadding=3)
-        return t, (accentry, nameentry, mailentry, replyentry, sigentry,
-                   accname)
+
+        t.attach(la, 0, 1, 0, 1, yoptions=0, ypadding=3, xpadding=10)
+        t.attach(accentry, 0, 1, 1, 2, xpadding=10, yoptions=0, ypadding=3)
+        t.attach(ln, 0, 1, 2, 3, yoptions=0, ypadding=3, xpadding=10)
+        t.attach(nameentry, 0, 1, 3, 4, xpadding=10, yoptions=0, ypadding=3)
+        t.attach(le, 0, 1, 4, 5, yoptions=0, ypadding=3, xpadding=10)
+        t.attach(mailentry, 0, 1, 5, 6, xpadding=10, yoptions=0, ypadding=3)
+        t.attach(lr, 0, 1, 6, 7, yoptions=0, ypadding=3, xpadding=10)
+        t.attach(replyentry, 0, 1, 7, 8, xpadding=10, yoptions=0, ypadding=3)
+        t.attach(ls, 0, 1, 8, 9, yoptions=0, ypadding=3, xpadding=10)
+        t.attach(sigentry, 0, 1, 10, 11, xpadding=10, yoptions=0, ypadding=3)
+        frame.add(t)
+
+        # Make tab for outgoing
+        t = GtkTable(4,1,0)
+        t.show()
+        frame = GtkFrame('Outgoing')
+        frame.set_border_width(10)
+        frame.set_usize(320, 300)
+        frame.show()
+        label = GtkLabel('Outgoing')
+        notebook.append_page(frame, label)
+
+        bsv = GtkRadioButton(None, "Send using mail server:")
+        bsv.show()
+        bsv.set_active ( usesmtpserver )
+	serverentry = GtkEntry()
+        serverentry.show()
+        if smtpserver != '':
+            serverentry.set_text (smtpserver)
+
+        bsm = GtkRadioButton(bsv, "Send using local 'sendmail':")
+        bsm.show()
+        bsm.set_active ( not usesmtpserver )
+
+	sendmailentry = GnomeFileEntry()
+        sendmailentry.gnome_entry().set_history_id('sendmail')
+        sendmailentry.gnome_entry().load_history()
+        sendmailentry.show()
+        sendmailentry.gtk_entry().set_text(sendmail)
+
+        t.attach(bsv, 0, 1, 0, 1, yoptions=0, ypadding=3, xpadding=10)
+        t.attach(serverentry, 0, 1, 1, 2, xpadding=10, yoptions=0, ypadding=3)
+        t.attach(bsm, 0, 1, 3, 4, yoptions=0, ypadding=3, xpadding=10)
+        t.attach(sendmailentry, 0, 1, 4, 5, xpadding=10, yoptions=0, ypadding=3)
+        frame.add(t)
+
+        # Return notebook
+        return notebook, (accentry, nameentry, mailentry, replyentry, sigentry,
+                          serverentry, sendmailentry, bsv, accname)
 
 
     def set_as_default(self, button):
@@ -593,33 +669,6 @@ class PreferencesWindow:
         self.update_acc()
 
 
-    ## Create the 'Outgoing' notebook
-    def init_outgoing(self, frame):
-        t = GtkTable(8,1,0)
-        t.show()
-	self.usemailserver = GtkRadioButton(None, "Send using mail server:")
-	self.usemailserver.show()
-        self.usemailserver.set_active(self.prefs.usemailserver)
-	self.serverentry = GtkEntry()
-        self.serverentry.show()
-        self.serverentry.set_text(self.prefs.serverentry)
-        self.usesendmail = GtkRadioButton(self.usemailserver,
-                                          "Send using local 'sendmail':")
-	self.usesendmail.show()
-        self.usesendmail.set_active(self.prefs.usesendmail)
-	self.sendmailentry = GnomeFileEntry()
-        self.sendmailentry.gnome_entry().set_history_id('sendmail')
-        self.sendmailentry.gnome_entry().load_history()
-        self.sendmailentry.show()
-        self.sendmailentry.gtk_entry().set_text(self.prefs.sendmailentry)
-        t.attach(self.usemailserver, 0, 1, 0, 1, yoptions=0, ypadding=3, xpadding=10)
-        t.attach(self.serverentry, 0, 1, 1, 2, xpadding=10, yoptions=0,
-                 ypadding=3)
-        t.attach(self.usesendmail, 0, 1, 2, 3, yoptions=0, ypadding=3, xpadding=10)
-        t.attach(self.sendmailentry, 0, 1, 3, 4, xpadding=10, yoptions=0,
-                 ypadding=3)
-        frame.add(t)
-
     ## Create the 'Editor' notebook
     def init_editor(self, frame):
 	from externaleditor import hintlist
